@@ -4,7 +4,7 @@
 
 ;; Author: João Távora <joaotavora@gmail.com>
 ;; Package-Requires: ((emacs "28.1") (project "0.9.8"))
-;; Version: 1.0.0
+;; Version: 1.0.1
 ;; Keywords:
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -21,78 +21,78 @@
 ;; along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
-;;;
+;;
 ;;;; Usage:
-;;;
-;;; Breadcrumbs are sequences of short strings indicating where you
-;;; are in some big tree-like maze.
-;;;
-;;; To craft these strings, this library uses the maps provided by
-;;; project.el and Imenu, respectively.  Project breadcrumbs shows you
-;;; the current buffer's path in a large project.  Imenu breadcrumbs
-;;; show the current position of point in the buffer's nested
-;;; structure of programming constructs (for example, a specific
-;;; functions within multiple C++ nested namespaces).
-;;;
-;;; To use this library:
-;;;
-;;; * `M-x breadcrumb-mode` is a global mode.  Will try to turn itself
-;;;   on conservatively and only if there's a project.
-
-;;; * `M-x breadcrumb-local-mode` is a buffer-local minor mode, if you
-;;;    don't want the default heuristics for turning it on everywhere.
-;;;
-;;; * Manually put the mode-line constructs
-;;;
-;;;     (:eval (breadcrumb-imenu-crumbs))
-;;;
-;;;   and
-;;;
-;;;     (:eval (breadcrumb-project-crumbs))
-;;;
-;;;  in your settings of the `mode-line-format' or
-;;;  `header-line-format' variables.
-;;;
-;;; The shape and size of each breadcrumb groups may be tweaked via
-;;; `breadcrumb-imenu-max-length', `breadcrumb-project-max-length',
-;;; `breadcrumb-imenu-crumb-separator', and
-;;; `breadcrumb-project-crumb-separator'.
-;;;
-;;; The structure each the breadcrumbs varies depending on whether
-;;; either project.el and imenu.el (or both) can do useful things for
-;;; your buffer.
-;;;
-;;; For Project breadcrumbs, this depends on whether project.el's
-;;; `project-current' can guess what project the current buffer
-;;; belongs to.
-;;;
-;;; For Imenu breadcrumbs, this varies.  Depending on the major-mode
-;;; author's taste, the Imenu tree (in variable `imenu--index-alist')
-;;; may have different structure.  Sometimes, minor mode also tweak
-;;; the Imenu tree in useful ways.  For example, with recent Eglot (I
-;;; think Eglot 1.14+), managed buffers get extra region info added to
-;;; it, which makes Breadcrumb show "richer" paths.
-;;;
+;;
+;; Breadcrumbs are sequences of short strings indicating where you
+;; are in some big tree-like maze.
+;;
+;; To craft these strings, this library uses the maps provided by
+;; project.el and Imenu, respectively.  Project breadcrumbs shows you
+;; the current buffer's path in a large project.  Imenu breadcrumbs
+;; show the current position of point in the buffer's nested
+;; structure of programming constructs (for example, a specific
+;; functions within multiple C++ nested namespaces).
+;;
+;; To use this library:
+;;
+;; * `M-x breadcrumb-mode` is a global mode.  Will try to turn itself
+;;   on conservatively and only if there's a project.
+;;
+;; * `M-x breadcrumb-local-mode` is a buffer-local minor mode, if you
+;;    don't want the default heuristics for turning it on everywhere.
+;;
+;; * Manually put the mode-line constructs
+;;
+;;     (:eval (breadcrumb-imenu-crumbs))
+;;
+;;   and
+;;
+;;     (:eval (breadcrumb-project-crumbs))
+;;
+;;  in your settings of the `mode-line-format' or
+;;  `header-line-format' variables.
+;;
+;; The shape and size of each breadcrumb groups may be tweaked via
+;; `breadcrumb-imenu-max-length', `breadcrumb-project-max-length',
+;; `breadcrumb-imenu-crumb-separator', and
+;; `breadcrumb-project-crumb-separator'.
+;;
+;; The structure each the breadcrumbs varies depending on whether
+;; either project.el and imenu.el (or both) can do useful things for
+;; your buffer.
+;;
+;; For Project breadcrumbs, this depends on whether project.el's
+;; `project-current' can guess what project the current buffer
+;; belongs to.
+;;
+;; For Imenu breadcrumbs, this varies.  Depending on the major-mode
+;; author's taste, the Imenu tree (in variable `imenu--index-alist')
+;; may have different structure.  Sometimes, minor mode also tweak
+;; the Imenu tree in useful ways.  For example, with recent Eglot (I
+;; think Eglot 1.14+), managed buffers get extra region info added to
+;; it, which makes Breadcrumb show "richer" paths.
+;;
 ;;;; Implementation notes:
-;;;
-;;; This _should_ be faster than which-func.el due some caching
-;;; strategies.  One of these strategies occurs in `bc--ipath-alist',
-;;; which takes care not to over-call `imenu--make-index-alist', which
-;;; could be slow (in fact very slow if an external process needs to
-;;; be contacted).  The variable `breadcrumb-idle-delay' controls
-;;; that.  Another cache occurs in `bc--ipath-plain-cache' second is
-;;; just a simple "space-for-speed" cache.
-;;;
-;;; Breadcrumb uses the double-dashed Imenu symbols
-;;; `imenu--index-alist' and `imenu--make-index-alist'.  There's
-;;; really no official API here.  It's arguable that, despite the
-;;; name, these aren't really internal symbols (the much older
-;;; which-func.el library makes liberal use of them, for example).
-;;;
+;;
+;; This _should_ be faster than which-func.el due some caching
+;; strategies.  One of these strategies occurs in `bc--ipath-alist',
+;; which takes care not to over-call `imenu--make-index-alist', which
+;; could be slow (in fact very slow if an external process needs to
+;; be contacted).  The variable `breadcrumb-idle-delay' controls
+;; that.  Another cache occurs in `bc--ipath-plain-cache' second is
+;; just a simple "space-for-speed" cache.
+;;
+;; Breadcrumb uses the double-dashed Imenu symbols
+;; `imenu--index-alist' and `imenu--make-index-alist'.  There's
+;; really no official API here.  It's arguable that, despite the
+;; name, these aren't really internal symbols (the much older
+;; which-func.el library makes liberal use of them, for example).
+;;
 ;;;; Todo:
-;;;
-;;; Make more clicky buttons in the headerline to do whatever
-;;;
+;;
+;; Make more clicky buttons in the headerline to do whatever
+;;
 
 ;;; Code:
 (require 'cl-lib)
@@ -100,21 +100,30 @@
 (require 'project)
 
 
-;;; Customization options
+;;;; Customization options
+;;
 
 (defgroup breadcrumb nil
   "One-liner indication of where you are in the maze."
   :prefix "breadcrumb-"
   :group 'convenience)
 
-(defcustom bc-project-max-length 40
-  "Soft cutoff for `breadcrumb-project-crumbs'." :type 'natnum)
+(defcustom bc-project-max-length 0.3
+  "Soft cutoff for `breadcrumb-project-crumbs'.
+If a fixnum, it's a absolute number of characters.  If a float, a
+percentage of `window-width'."
+  :type '(choice (natnum :tag "Number of characters")
+                 (float  :tag "Percent of window's width")))
 
 (defcustom bc-project-crumb-separator "/"
   "Separator for `breadcrumb-project-crumbs'." :type 'string)
 
-(defcustom bc-imenu-max-length 40
-  "Soft cutoff for `breadcrumb-imenu-crumbs'." :type 'natnum)
+(defcustom bc-imenu-max-length 0.3
+  "Soft cutoff for `breadcrumb-imenu-crumbs'.
+If a fixnum, it's a absolute number of characters.  If a float, a
+percentage of `window-width'."
+  :type '(choice (natnum :tag "Number of characters")
+                 (float  :tag "Percent of window's width")))
 
 (defcustom bc-imenu-crumb-separator " > "
   "Separator for `breadcrumb-project-crumbs'." :type 'string)
@@ -139,8 +148,8 @@
   "Face for the project leaf crumb in breadcrumb project path.")
 
 
-;;; "ipath" management logic and imenu interoperation
-;;;
+;;;; "ipath" management logic and imenu interoperation
+;;
 (cl-defun bc--bisect (a x &key (from 0) (to (length a)) key from-end)
   "Compute index to insert X in sequence A, keeping it sorted.
 If X already in A, the resulting index is the leftmost such
@@ -184,10 +193,24 @@ These structures don't have a `breadcrumb-region' property on."
                 (setq ipath (cons (car n) ipath))
                 (if (consp (cdr n))
                     (mapc (lambda (n2) (dfs n2 ipath (cdr n))) (cdr n))
+                  ;; FIXME: we convert markers to points via the `+'
+                  ;; down there.  But for siblings, no such conversion
+                  ;; happens, they might still point to invalid
+                  ;; markers.  Not worth doing another tree traversal
+                  ;; for that IMO, and siblings seems to be unused
+                  ;; anyway (github#22)
                   (put-text-property 0 1 'breadcrumb-siblings (cdr siblings) (car ipath))
                   (setq bc--ipath-plain-cache
                         (vconcat bc--ipath-plain-cache
-                                 `[,(cons (cdr n) ipath)])))))
+                                 `[,(cons
+                                     ;; See github#17 and docstring of
+                                     ;; `imenu--index-alist' for the
+                                     ;; "overlay" edge case.
+                                     (cl-etypecase (cdr n)
+                                       (number (cdr n))
+                                       (marker (+ (cdr n) 0))
+                                       (overlay (overlay-start (cdr n))))
+                                     ipath)])))))
     (unless bc--ipath-plain-cache
       (mapc (lambda (i) (dfs i nil index-alist)) index-alist)
       (setq bc--ipath-plain-cache (cl-sort bc--ipath-plain-cache #'< :key #'car)))
@@ -236,13 +259,19 @@ These structures don't have a `breadcrumb-region' property on."
     imenu--index-alist))
 
 
-;;; Higher-level functions
+;;;; Higher-level functions
+;;
 
 ;; FIXME: Why do I need to put these key definitiosn in special
 ;; variables?
 (defvar bc--header-line-key [header-line mouse-1])
 (defvar bc--mode-line-key [mode-line mouse-1])
 
+(defun bc--length (len)
+  "Interpret LEN using `window-width' and return a number."
+  (cond ((floatp len) (* (window-width) len))
+        (t len)))
+
 (defun bc--format-ipath-node (p more)
   (let* ((l (lambda (&rest _event)
               (interactive)
@@ -256,7 +285,7 @@ These structures don't have a `breadcrumb-region' property on."
      p 'mouse-face 'header-line-highlight
      'face (if more 'bc-imenu-crumbs-face 'bc-imenu-leaf-face)
      'bc-dont-shorten (null more)
-     'help-echo (format "mouse-1: Go places near %p" p)
+     'help-echo (format "mouse-1: Go places near %s" p)
      'keymap
      (let ((m (make-sparse-keymap)))
        (define-key m bc--header-line-key l)
@@ -272,27 +301,27 @@ These structures don't have a `breadcrumb-region' property on."
        (cl-loop
         for (p . more) on (bc-ipath alist (point))
         collect (bc--format-ipath-node p more))
-       bc-imenu-max-length
+       (bc--length bc-imenu-max-length)
        (propertize bc-imenu-crumb-separator
                    'face 'bc-face)))))
 
 (defun bc--summarize (crumbs cutoff separator)
   "Return a string that summarizes CRUMBS, a list of strings.
-\"Summarization\" consists of truncating some CRUMBS to 1
-character.  Rightmost members of CRUMBS are summarized last.
-Members with a `breadcrumb-dont-shorten' are never truncated.
-Aim for a return string that is at most CUTOFF characters long.
-Join the crumbs with SEPARATOR."
+\"Summarization\" consists of truncating some CRUMBS to 1 character.
+Rightmost members of CRUMBS are summarized last.  Members with a non-nil
+`breadcrumb-dont-shorten' property are never truncated.  Aim for a
+return string that is at most CUTOFF characters long.  Join the crumbs
+with SEPARATOR."
   (let ((rcrumbs
          (cl-loop
-             for available = (- cutoff used)
-             for (c . more) on (reverse crumbs)
-             for seplen = (if more (length separator) 0)
-             for shorten-p = (unless (get-text-property 0 'bc-dont-shorten c)
-                               (> (+ (length c) seplen) available))
-             for toadd = (if shorten-p (substring c 0 1) c)
-             sum (+ (length toadd) seplen) into used
-             collect toadd)))
+          for available = (- cutoff used)
+          for (c . more) on (reverse crumbs)
+          for seplen = (if more (length separator) 0)
+          for shorten-p = (unless (get-text-property 0 'bc-dont-shorten c)
+                            (> (+ (length c) seplen) available))
+          for toadd = (if shorten-p (substring c 0 1) c)
+          sum (+ (length toadd) seplen) into used
+          collect toadd)))
     (string-join (reverse rcrumbs) separator)))
 
 (defun bc--format-project-node (p more root path)
@@ -308,7 +337,7 @@ to ROOT."
                 (if more 'bc-project-crumbs-face 'bc-project-leaf-face)
                 'bc-dont-shorten (null more)
                 'mouse-face 'header-line-highlight
-                'help-echo (format "mouse-1: Go places near %s/%s" root path)
+                'help-echo (format "mouse-1: Go places near %s%s" root path)
                 'keymap
                 (let ((m (make-sparse-keymap)))
                   (define-key m bc--header-line-key l)
@@ -330,8 +359,8 @@ propertized crumbs."
    collect (bc--format-project-node s more root upto) into retval
    finally
    (cl-return
-    (if project
-        (cons (propertize (project-name project)
+    (if root
+        (cons (propertize (file-name-nondirectory (directory-file-name root))
                           'bc-dont-shorten t
                           'face 'bc-project-base-face)
               retval)
@@ -342,8 +371,10 @@ propertized crumbs."
   "Describing the current file inside project."
   (bc--summarize
    (if buffer-file-name (bc--project-crumbs-1 buffer-file-name)
-     (list (propertize (buffer-name) 'face 'bc-project-leaf-face)))
-   bc-project-max-length
+     (list (propertize (buffer-name)
+                       'face 'bc-project-leaf-face
+                       'bc-dont-shorten t)))
+   (bc--length bc-project-max-length)
    (propertize bc-project-crumb-separator
                'face 'bc-project-crumbs-face)))
 
@@ -358,7 +389,7 @@ propertized crumbs."
 (define-minor-mode breadcrumb-local-mode
   "Header lines with breadcrumbs."
   :init-value nil
- (if bc-local-mode (add-to-list 'header-line-format '(:eval (bc--header-line)))
+  (if bc-local-mode (add-to-list 'header-line-format '(:eval (bc--header-line)))
     (setq header-line-format (delete '(:eval (bc--header-line)) header-line-format))))
 
 (defun bc--turn-on-local-mode-on-behalf-of-global-mode ()
@@ -407,8 +438,11 @@ propertized crumbs."
       (bc--goto (selected-window) choice))))
 
 (provide 'breadcrumb)
-;;; breadcrumb.el ends here
+;;;###autoload (register-definition-prefixes "breadcrumb" '("breadcrumb-"))
 
 ;; Local Variables:
 ;; read-symbol-shorthands: (("bc-" . "breadcrumb-"))
+;; autoload-compute-prefixes: nil
 ;; End:
+
+;;; breadcrumb.el ends here
